package com.hikcreate.controller;

import com.alibaba.fastjson.JSONObject;
import com.hikcreate.drv_photo_pic.DrvPhoto;
import com.hikcreate.drv_photo_pic.Vehicle;
import com.hikcreate.entity.PicByte;
import com.hikcreate.service.fdfs.service.FileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import javax.imageio.stream.FileImageOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Map;

import org.springframework.http.*;


/**
 * @author lifuyi
 * @date ${Date}
 * @description
 */
@RestController
@RequestMapping("/pic")
public class PicController {


    @Autowired
    private DrvPhoto drvPhoto;
    @Autowired
    private FileService fileService;
    @Value("${ftpUrl}")
    private String ftpUrl;

    @Value("${httpUrl}")
    private String httpUrl;

    @Autowired
    private Vehicle vehicle;

    /**
     * 插入指定贵阳驾驶人头像
     */
    @GetMapping("/insertDrvPhotoById")
    public void insertDrvPhotoById(@RequestParam("sfzmhm") String sfzmhm) {
        drvPhoto.insertDrvPhotoById(sfzmhm);
    }

    /**
     * 车辆历史图片
     *
     * @param time
     */
    @GetMapping("/getHisVehPic")
    public void getHisVehPic(@RequestParam("time") String time) {
        vehicle.getHisVehPic(time);
    }

    /**
     * 同步指定时间段车辆图片数据
     */
    @GetMapping("/getVehPicStage")
    public void getVehPicStage(@RequestParam("start") String start, @RequestParam("end") String end) {
        vehicle.getVehPicStage(start, end);
    }


    /**
     * 车辆增量图片
     */
    @GetMapping("/getIncrementVehPic")
    public void getIncrementVehPic() {
        vehicle.getIncrementVehPic();
    }

    /**
     * 根据车牌号和车牌类型插入图片
     */
    @GetMapping("/insertVehPicByCar")
    @ResponseBody
    public ResponseEntity insertVehPicByCar(@RequestParam("hphm") String hphm, @RequestParam("hpzl") String hpzl) {
        vehicle.insertVehPicByCar(hphm, hpzl);
        return new ResponseEntity("sucess", HttpStatus.OK);
    }

    /**
     * 根据序号插入车辆图片
     *
     * @param xh
     */
    @GetMapping("/insertVehPicByXh")
    public void insertVehPicByXh(@RequestParam("xh") String xh) {
        vehicle.insertVehPicByXh(xh);
    }

    /**
     * 指定时间段驾驶人头像同步
     *
     * @param start
     * @param end
     */
    @GetMapping("/increDrvPic")
    public void increDrvPic(@RequestParam("start") String start, @RequestParam("end") String end) {
        drvPhoto.increDrvPic(start, end);
    }

    /**
     * 根据地址删除图片
     * @param url
     */
    @GetMapping("/deletePicByUrl")
    public void deletePicByUrl(@RequestParam("url") String url) {
        try {
            fileService.deleteFile(url);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 删除Hbase机动车表所有数据
     * @param table
     * @param maxrow
     * @param minrow
     */
    @GetMapping("/deleteAllVehPic")
    public void deleteAllVehPic(@RequestParam("table") String table,@RequestParam("maxrow") String maxrow,@RequestParam("minrow") String minrow){
        vehicle.deleteAllVehPic(table,maxrow,minrow);
    }


    @GetMapping("/drvPhotoHis")
    public void getHisDrvPhoto(@RequestParam("time") String time, HttpServletResponse response) {
        try {
            boolean result = drvPhoto.getHisDrvPhoto(time);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("result", result);
            response.setContentType("application/json;charset=UTF-8");
            response.getWriter().write(jsonObject.toJSONString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * app端通过此接口获取违法图片
     * @param url
     * @return
     */
    @GetMapping("/getVioPic")
    public synchronized PicByte getVioPicByUrl(@RequestParam("url") String url) {
        PicByte picByte = new PicByte();
        RestTemplate restTemplate = new RestTemplate();
        MultiValueMap<String, String> params = new LinkedMultiValueMap<String, String>();
        params.add("urls", url);
        HttpHeaders headers = new HttpHeaders();
        HttpMethod method = HttpMethod.POST;
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<>(params, headers);
        Map<String, byte[]> map;
        if (url.startsWith("ftp")) {
            ResponseEntity<PicByte> resp = restTemplate.exchange(ftpUrl, method, requestEntity, PicByte.class);
            map = resp.getBody().getMap();
        } else if (url.startsWith("http")) {
            ResponseEntity<PicByte> resp = restTemplate.exchange(httpUrl, method, requestEntity, PicByte.class);
            map = resp.getBody().getMap();
        } else {
            return null;
        }
        picByte.setMap(map);
        return picByte;
    }


    @GetMapping("/testHttp")
    public void testHttp(@RequestParam("param") String param, HttpServletResponse response) {
        try {
            RestTemplate restTemplate = new RestTemplate();
            MultiValueMap<String, String> params = new LinkedMultiValueMap<String, String>();
            params.add("urls", param);
            HttpHeaders headers = new HttpHeaders();
            HttpMethod method = HttpMethod.POST;
            headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
            HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<>(params, headers);
            ResponseEntity<PicByte> resp = restTemplate.exchange(ftpUrl, method, requestEntity, PicByte.class);
            Map<String, byte[]> map = resp.getBody().getMap();
            //遍历Map
            String path = "/home/pic/";
            File file;
            for (Map.Entry<String, byte[]> entry : map.entrySet()) {
                String[] split = entry.getKey().split("/");
                file = new File(path + split[split.length - 1]);
                if (file.exists()) {
                    file.delete();
                }
                FileImageOutputStream imageOutput = new FileImageOutputStream(file);
                imageOutput.write(entry.getValue(), 0, entry.getValue().length);
                imageOutput.close();
            }
            response.setContentType("application/json;charset=UTF-8");
            response.getWriter().write(path);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
