package com.hikcreate.drv_photo_pic.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.hikcreate.drv_photo_pic.DrvPhoto;
import com.hikcreate.entity.PicResult;
import com.hikcreate.service.fdfs.service.FileService;
import com.hikcreate.utils.DateUtil;
import oracle.sql.BLOB;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.sql.*;
import java.sql.Connection;
import java.util.zip.CRC32;


/**
 * @author lifuyi
 * @date ${Date}
 * @description
 */
@Service("drvPhotoImpl")
public class DrvPhotoImpl implements DrvPhoto {


    private static Logger logger = LoggerFactory.getLogger(DrvPhotoImpl.class);
    @Value("${url}")
    private String url;
    @Value("${username}")
    private String username;
    @Value("${password}")
    private String password;
    @Value("${roundDay}")
    private int roundDay;
    @Autowired
    private FileService fileService;

    private static String startTime;
    private static String endTime;
    private static String sfzmhm;
    private static BLOB zp;
    private static String gxsj;
    private static String xzqh;
    private static String xh;
    private static String flag;
    private static String rksj;
    private static String fzjg;


    private static Configuration hbaseConf;
    private static org.apache.hadoop.hbase.client.Connection hbaseConn;
    private static TableName driverPhotoTablename;
    private static Table driverPhototable;
    @Value("${hbase.zookeeper.property.clientPort}")
    private String hbaseAddress;
    @Value("${hbase.zookeeper.quorum}")
    private String hbaseZkQuorum;
    @Value("${hbase.master}")
    private String hbaseMaster;
    @Value("${hbase.drv_photo.table}")
    private String driverPhotoTableStr;
    private static String[] colValue;

    private static CRC32 crc = new CRC32();

    /**
     * 同步历史图片，图片同步到time时间
     *
     * @param time
     * @return
     */
    public boolean getHisDrvPhoto(String time) {
        hbaseConf = HBaseConfiguration.create();
        hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
        hbaseConf.set("hbase.zookeeper.quorum", hbaseZkQuorum);
        try {
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            endTime = time;
            startTime = SqlHelp.getStartTime("select  min(GXSJ) mtime from GYJG.DRV_PHOTO", url, username, password);
            //循环获取图片
            String sql;
            String lastTime;
            driverPhotoTablename = TableName.valueOf(driverPhotoTableStr);
            driverPhototable = hbaseConn.getTable(driverPhotoTablename);
            while (endTime.compareTo(startTime) > 0 || endTime.compareTo(startTime) == 0) {
                lastTime = DateUtil.formatDate(DateUtil.getPastDate(DateUtil.parseDate(endTime), roundDay));
                Connection connection = DriverManager.getConnection(url, username, password);
                sql = "select * from GYJG.DRV_PHOTO where to_char(GXSJ,'yyyy-mm-dd')>='" + lastTime + "' and to_char(GXSJ,'yyyy-mm-dd')<'"
                        + endTime + "' and ZP is not null and fzjg is not null and sfzmhm is not null ";
                PreparedStatement pstm = connection.prepareStatement(sql);
                ResultSet resultSet = pstm.executeQuery();
                while (resultSet.next()) {
                    saveToHbase(resultSet, fileService);
                }
                endTime = lastTime;
                resultSet.close();
                pstm.close();
                connection.close();
                Thread.sleep(1000);
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            closeTable();
        }
        return true;
    }

    /**
     * 每天增量同步
     */
    @Override
    public void getIncrementDrvPhoto() {
        hbaseConf = HBaseConfiguration.create();
        hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
        hbaseConf.set("hbase.zookeeper.quorum", hbaseZkQuorum);
        try {
            Class.forName("oracle.jdbc.OracleDriver");
            Connection connection = DriverManager.getConnection(url, username, password);
            String sql = "select * from GYJG.DRV_PHOTO where GXSJ between sysdate-3 and sysdate and ZP is not null and fzjg is not null and sfzmhm is not null";
            PreparedStatement pstm = connection.prepareStatement(sql);
            ResultSet resultSet = pstm.executeQuery();
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            driverPhotoTablename = TableName.valueOf(driverPhotoTableStr);
            driverPhototable = hbaseConn.getTable(driverPhotoTablename);
            while (resultSet.next()) {
                saveToHbase(resultSet, fileService);
            }
            resultSet.close();
            pstm.close();
            connection.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            closeTable();
        }
    }

    /**
     * 插入指定驾驶人头像
     *
     * @param sfzmhm
     */
    @Override
    public void insertDrvPhotoById(String sfzmhm) {
        hbaseConf = HBaseConfiguration.create();
        hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
        hbaseConf.set("hbase.zookeeper.quorum", hbaseZkQuorum);
        try {
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            driverPhotoTablename = TableName.valueOf(driverPhotoTableStr);
            driverPhototable = hbaseConn.getTable(driverPhotoTablename);
            String sql = "select * from GYJG.DRV_PHOTO where  ZP is not null and sfzmhm=? order by gxsj";
            Connection connection = DriverManager.getConnection(url, username, password);
            PreparedStatement pstm = connection.prepareStatement(sql);
            pstm.setString(1, sfzmhm);
            ResultSet resultSet = pstm.executeQuery();
            while (resultSet.next()) {
                saveToHbase(resultSet, fileService);
            }
            resultSet.close();
            pstm.close();
            connection.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            closeTable();
        }
    }

    @Override
    public void increDrvPic(String start, String end) {
        hbaseConf = HBaseConfiguration.create();
        hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
        hbaseConf.set("hbase.zookeeper.quorum", hbaseZkQuorum);
        try {
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            driverPhotoTablename = TableName.valueOf(driverPhotoTableStr);
            driverPhototable = hbaseConn.getTable(driverPhotoTablename);
            String sql = "select * from GYJG.DRV_PHOTO where   ZP is not null and fzjg='贵A' and sfzmhm is not null " +
                    "and to_char(GXSJ,'yyyy-mm-dd')>='" + start + "' and to_char(GXSJ,'yyyy-mm-dd')<'" + end + "'";
            Connection connection = DriverManager.getConnection(url, username, password);
            PreparedStatement pstm = connection.prepareStatement(sql);
            ResultSet resultSet = pstm.executeQuery();
            while (resultSet.next()) {
                saveToHbase(resultSet, fileService);
            }
            resultSet.close();
            pstm.close();
            connection.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            closeTable();
        }
    }

    private synchronized static void saveToHbase(ResultSet resultSet, FileService fileService) throws Exception {
        paramResultSet(resultSet);
        //判断图片是否已经存入fastdfs
        colValue = new String[2];
        String rowkey = sfzmhm;
        PicResult picResult = SqlHelp.getFlag(rowkey, driverPhototable, colValue, gxsj);
        if (picResult.getI() != 0) {
            if (picResult.getI() == 2) {
                fileService.deleteFile(picResult.getUrl());
            }
            Put put = new Put(rowkey.getBytes());
            put.addColumn("info".getBytes(), "sfzmhm".getBytes(), sfzmhm.getBytes());
            byte[] bytes1 = SqlHelp.blobToByteZp(zp);
            String picUrl = fileService.uploadFile(bytes1, sfzmhm + ".jpeg");
            put.addColumn("info".getBytes(), "sfzmhm".getBytes(), (sfzmhm == null ? "null" : sfzmhm).getBytes());
            put.addColumn("info".getBytes(), "picUrl".getBytes(), picUrl.getBytes());
            put.addColumn("info".getBytes(), "gxsj".getBytes(), (gxsj == null ? "null" : gxsj).getBytes());
            put.addColumn("info".getBytes(), "xzqh".getBytes(), (xzqh == null ? "null" : xzqh).getBytes());
            put.addColumn("info".getBytes(), "flag".getBytes(), (flag == null ? "null" : flag).getBytes());
            put.addColumn("info".getBytes(), "xh".getBytes(), (xh == null ? "null" : xh).getBytes());
            put.addColumn("info".getBytes(), "rksj".getBytes(), (rksj == null ? "null" : rksj).getBytes());
            put.addColumn("info".getBytes(), "fzjg".getBytes(), (fzjg == null ? "null" : fzjg).getBytes());
            driverPhototable.put(put);
        }
    }


    public static String getPrefixString(long prefixLong) {
        if (prefixLong < 10L) {
            return "00" + prefixLong;
        } else if (prefixLong >= 10L && prefixLong < 100L) {
            return "0" + prefixLong;
        } else {
            return "" + prefixLong;
        }
    }

    private void closeTable() {
        if (driverPhototable != null) {
            try {
                driverPhototable.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if (hbaseConn != null) {
            try {
                hbaseConn.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private synchronized static void paramResultSet(ResultSet resultSet) throws SQLException {
        sfzmhm = resultSet.getString("SFZMHM");
        zp = (BLOB) resultSet.getBlob("ZP");
        gxsj = resultSet.getString("GXSJ");
        xzqh = resultSet.getString("XZQH");
        flag = resultSet.getString("FLAG");
        xh = resultSet.getString("XH");
        rksj = resultSet.getString("RKSJ");
        fzjg = resultSet.getString("FZJG");
    }

    public static void main(String[] args) {
        String url = "ftp://vion6:vion6@52.1.113.218:21/193.2.117.66/kk/2019-07-06/18/2019070618541777800022598.jpg";
        String[] arr = url.split("/");
        System.out.println(arr[arr.length - 1].split("jpg")[0] + "jpg");
    }

}
