package com.hikcreate.drv_photo_pic.impl;

import com.hikcreate.drv_photo_pic.Vehicle;
import com.hikcreate.entity.PicResult;
import com.hikcreate.service.fdfs.service.FileService;
import com.hikcreate.utils.DateUtil;
import oracle.sql.BLOB;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.*;
import org.apache.hadoop.hbase.util.Bytes;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.sql.*;
import java.sql.Connection;
import java.util.Iterator;
import java.util.Map;

@Service("vehicleImpl")
public class VehicleImpl implements Vehicle {

    private static Logger logger = LoggerFactory.getLogger(VehicleImpl.class);
    @Autowired
    private FileService fileService;

    @Value("${hbase.veh_pic.table}")
    private String vehPicTableStr;
    @Value("${hbase.zookeeper.property.clientPort}")
    private String hbaseAddress;
    @Value("${hbase.zookeeper.quorum}")
    private String hbaseZkQuorum;
    @Value("${url}")
    private String url;
    @Value("${username}")
    private String username;
    @Value("${password}")
    private String password;
    @Value("${roundDay}")
    private int roundDay;

    private static String startTime;
    private static String endTime;
    private static String[] colValue;


    /**
     * 全量车辆图片
     *
     * @param time
     */
    @Override
    public void getHisVehPic(String time) {
        org.apache.hadoop.hbase.client.Connection hbaseConn = null;
        Table vehPicTable = null;
        try {
            Configuration hbaseConf = HBaseConfiguration.create();
            hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
            hbaseConf.set("hbase.zookeeper.quorum", hbaseZkQuorum);
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            TableName vehPicTableName = TableName.valueOf(vehPicTableStr);
            vehPicTable = hbaseConn.getTable(vehPicTableName);
            endTime = time;
            startTime = SqlHelp.getStartTime("select  min(GXSJ) mtime from GYJG.VEH_PICTURE", url, username, password);
            String lastTime;
            String sql;
            while (endTime.compareTo(startTime) > 0 || endTime.compareTo(startTime) == 0) {
                Connection connection = DriverManager.getConnection(url, username, password);
                lastTime = DateUtil.formatDate(DateUtil.getPastDate(DateUtil.parseDate(endTime), roundDay));
                sql = "select * from GYJG.VEH_PICTURE where to_char(GXSJ,'yyyy-mm-dd')>=? and to_char(GXSJ,'yyyy-mm-dd')<? " +
                        " and ZP is not null and XH is not null ";
                PreparedStatement pstm = connection.prepareStatement(sql);
                pstm.setString(1, lastTime);
                pstm.setString(2, endTime);
                ResultSet resultSet = pstm.executeQuery();
                while (resultSet.next()) {
                    saveToHbase(resultSet, vehPicTable);
                }
                endTime = lastTime;
                resultSet.close();
                connection.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (hbaseConn != null) {
                try {
                    hbaseConn.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (vehPicTable != null) {
                try {
                    vehPicTable.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * 增量车辆图片
     */
    @Override
    public void getIncrementVehPic() {
        String sql = "select * from GYJG.VEH_PICTURE where GXSJ between sysdate-10 and sysdate and ZP is not null and XH is not null";
        getPic(sql);
    }

    /**
     * 根据车牌号获取图片
     *
     * @param hphm
     * @param hpzl
     */
    @Override
    public void insertVehPicByCar(String hphm, String hpzl) {
        String sql = "select b.XH xh,b.HPHM hphm,b.HPZL hpzl,b.ZP zp,b.GXSJ gxsj from ZCKJ.VM_VEHICLE_GY a inner join GYJG.VEH_PICTURE b " +
                "on a.XH=b.XH where a.HPHM='" + hphm + "' and a.HPZL='" + hpzl + "'";
        getPic(sql);

    }

    /**
     * 同步指定时间段的增量图片
     *
     * @param start
     * @param end
     */
    @Override
    public void getVehPicStage(String start, String end) {
        String sql = "select * from GYJG.VEH_PICTURE where " +
                "to_char(GXSJ,'yyyy-mm-dd')>'" + start + "' and to_char(GXSJ,'yyyy-mm-dd')<='" + end + "' and ZP is not null and XH is not null";
        getPic(sql);
    }

    /**
     * 根据序号插入图片
     *
     * @param xh
     */
    @Override
    public void insertVehPicByXh(String xh) {
        String sql = "select * from GYJG.VEH_PICTURE where XH='" + xh + "'";
        getPic(sql);
    }

    @Override
    public void deleteAllVehPic(String table1, String maxrow, String minrow) {
        org.apache.hadoop.hbase.client.Connection connection = null;
        Table table = null;
        try {
            Configuration conf = HBaseConfiguration.create();
            conf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
            conf.set("hbase.zookeeper.quorum", hbaseZkQuorum);
            connection = ConnectionFactory.createConnection(conf);
            table = connection.getTable(TableName.valueOf(table1));
            Scan scan = new Scan();
            scan.setStartRow(Bytes.toBytes(minrow));
            scan.setStopRow(Bytes.toBytes(maxrow));
            ResultScanner rs = table.getScanner(scan);
            Iterator<Result> it = rs.iterator();
            while (it.hasNext()) {
                Result r = it.next();
                Map<byte[], byte[]> map = r.getFamilyMap(Bytes.toBytes("info"));
                String url = Bytes.toString(map.get(Bytes.toBytes("picUrl")));
                try {
                    fileService.deleteFile(url);
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    Delete delete = new Delete(r.getRow());
                    table.delete(delete);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        finally {
            if(connection!=null){
                try {
                    connection.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if(table!=null){
                try {
                    table.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    private void getPic(String sql) {
        org.apache.hadoop.hbase.client.Connection hbaseConn = null;
        Table vehPicTable = null;
        try {
            Configuration hbaseConf = HBaseConfiguration.create();
            hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
            hbaseConf.set("hbase.zookeeper.quorum", hbaseZkQuorum);
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            TableName vehPicTableName = TableName.valueOf(vehPicTableStr);
            vehPicTable = hbaseConn.getTable(vehPicTableName);
            Class.forName("oracle.jdbc.OracleDriver");
            Connection connection = DriverManager.getConnection(url, username, password);
            PreparedStatement pstm = connection.prepareStatement(sql);
            ResultSet resultSet = pstm.executeQuery();
            while (resultSet.next()) {
                saveToHbase(resultSet, vehPicTable);
            }
            resultSet.close();
            pstm.close();
            connection.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (hbaseConn != null) {
                try {
                    hbaseConn.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (vehPicTable != null) {
                try {
                    vehPicTable.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    /**
     * 存储图片
     *
     * @param resultSet
     * @param vehPicTable
     */
    private synchronized void saveToHbase(ResultSet resultSet, Table vehPicTable) {
        try {
            String xh = resultSet.getString("XH");
            String hphm = resultSet.getString("HPHM");
            String hpzl = resultSet.getString("HPZL");
            BLOB zp = (BLOB) resultSet.getBlob("ZP");
            String gxsj = resultSet.getString("GXSJ");
            String rowkey = xh;
            colValue = new String[2];
            PicResult picResult = SqlHelp.getFlag(rowkey, vehPicTable, colValue, gxsj);
            if (picResult.getI() != 0) {
                if (picResult.getI() == 2) {
                    try {
                        fileService.deleteFile(picResult.getUrl());
                    } catch (Exception e) {
                        logger.info(xh + "---------------------" + picResult.getUrl());
                        e.printStackTrace();
                    } finally {
                        Delete delete = new Delete(rowkey.getBytes());
                        vehPicTable.delete(delete);
                    }
                }
                Put put = new Put(rowkey.getBytes());
                put.addColumn("info".getBytes(), "xh".getBytes(), (xh == null ? "null" : xh).getBytes());
                put.addColumn("info".getBytes(), "hphm".getBytes(), (hphm == null ? "null" : hphm).getBytes());
                put.addColumn("info".getBytes(), "hpzl".getBytes(), (hpzl == null ? "null" : hpzl).getBytes());
                put.addColumn("info".getBytes(), "gxsj".getBytes(), (gxsj == null ? "null" : gxsj).getBytes());
                byte[] bytes = SqlHelp.blobToByteZp(zp);
                String picUrl = uploadFile(bytes, xh + ".jpeg", xh);
                put.addColumn("info".getBytes(), "picUrl".getBytes(), picUrl.getBytes());
                vehPicTable.put(put);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    private String uploadFile(byte[] file, String fileName, String xh) {

        String url;

        try {
            url = fileService.uploadFile(file, fileName);
        } catch (Exception e) {
            logger.info("---------------" + xh);
            e.printStackTrace();
            url = uploadFile(file, fileName, xh);
        }
        return url;
//        int count=0;
//        while ("false".equals(url)){
//            url=fileService.uploadFile(file, fileName);
//            count++;
//            if(count>10){
//                logger.info("---------------" + xh);
//            }
//        }

    }


}
